@@ -120,7 +120,9 @@ run_old_e2e_tests() {
 run_key_mappings_tests() {
     # There is no keys generated for CryptoAuthLib yet.
     # This condition should be removed when the keys are generated for the CAL provider
-    if ! [[ "$PROVIDER_NAME" = "cryptoauthlib" ]]; then
+    # The v1.0.0 version of libts required for testing trusted-services does not generate keys
+    # in the form of 00*.psa_its. Hence we skip this test for the TS provider. 
+    if ! [[ "$PROVIDER_NAME" = "cryptoauthlib" || "$PROVIDER_NAME" = "trusted-service" ]]; then
         echo "Execute key mappings tests"
         RUST_BACKTRACE=1 cargo test $TEST_FEATURES --manifest-path ./e2e_tests/Cargo.toml key_mappings
     fi
@@ -154,14 +156,18 @@ setup_mappings() {
     fi
 }
 
-# Use the newest version of the Rust toolchain
+# Change rust toolchain version
+if [[ ! -z ${RUST_TOOLCHAIN_VERSION:+x} ]]; then
+	rustup override set ${RUST_TOOLCHAIN_VERSION}
+fi
+
 rustup update
-MSRV=1.66.0
 
 # Parse arguments
 NO_CARGO_CLEAN=
 NO_STRESS_TEST=
 PROVIDER_NAME=
+TEST_NEXT_BRANCH_TRACKING=
 CONFIG_PATH=$(pwd)/e2e_tests/provider_cfg/tmp_config.toml
 while [ "$#" -gt 0 ]; do
     case "$1" in
@@ -171,6 +177,9 @@ while [ "$#" -gt 0 ]; do
         --no-stress-test )
             NO_STRESS_TEST="True"
         ;;
+        --test-next-branch-tracking )
+            TEST_NEXT_BRANCH_TRACKING="True"
+        ;;
         mbed-crypto | pkcs11 | tpm | trusted-service | cryptoauthlib | all | cargo-check | on-disk-kim)
             if [ -n "$PROVIDER_NAME" ]; then
                 error_msg "Only one provider name must be given"
@@ -196,6 +205,9 @@ while [ "$#" -gt 0 ]; do
         coverage )
             PROVIDER_NAME=$1
         ;;
+        mismatcher )
+            PROVIDER_NAME=$1
+        ;;
         *)
             error_msg "Unknown argument: $1"
         ;;
@@ -203,6 +215,15 @@ while [ "$#" -gt 0 ]; do
     shift
 done
 
+if [ "$TEST_NEXT_BRANCH_TRACKING" ]; then
+    echo "Track next branches for parallaxsecond repositories"
+    python3 $(pwd)/utils/release_tracking.py $(pwd)/Cargo.toml $(pwd)/e2e_tests/Cargo.toml
+    next_branch_result=$?
+    if [ "$next_branch_result" -ne 0 ]; then
+        error_msg "Failed to track next branches of parallaxsecond repositories."
+    fi
+fi
+
 # Check if the PROVIDER_NAME was given.
 if [ -z "$PROVIDER_NAME" ]; then
     error_msg "a provider name needs to be given as input argument to that script."
@@ -210,6 +231,12 @@ fi
 
 trap cleanup EXIT
 
+if [ "$PROVIDER_NAME" = "mismatcher" ]; then
+    python3 $(pwd)/utils/dependency_cross_matcher.py --deps_dir $(pwd)
+
+    exit 0
+fi
+
 if [ "$PROVIDER_NAME" = "tpm" ] || [ "$PROVIDER_NAME" = "all" ] || [ "$PROVIDER_NAME" = "coverage" ]; then
 	# Copy the NVChip for previously stored state. This is needed for the key mappings test.
     cp /tmp/ondisk/NVChip .
@@ -248,13 +275,12 @@ fi
 git submodule update --init
 
 if [ "$PROVIDER_NAME" = "coverage" ]; then
-    rustup toolchain install ${MSRV}
     PROVIDERS="trusted-service mbed-crypto tpm pkcs11"
     EXCLUDES="fuzz/*,e2e_tests/*,src/providers/cryptoauthlib/*,src/authenticators/jwt_svid_authenticator/*"
     UNIT_TEST_FEATURES="unix-peer-credentials-authenticator,direct-authenticator"
     # Install tarpaulin
-    # TODO: Stop using the --version parameter when MSRV is upgraded.
-    cargo +${MSRV} install cargo-tarpaulin --version 0.26.1
+    # TODO: Stop using the --version parameter when MSRV is upgraded (>1.66.0)
+    cargo install cargo-tarpaulin --version 0.26.1 --locked
 
     mkdir -p reports
 
@@ -270,7 +296,7 @@ if [ "$PROVIDER_NAME" = "coverage" ]; then
         cp -r $(pwd)/e2e_tests/fake_mappings/* mappings
 
         # Start service
-        RUST_LOG=info cargo +${MSRV} tarpaulin --out Xml --forward --command build --exclude-files="$EXCLUDES" \
+        RUST_LOG=info cargo tarpaulin --out Xml --forward --command build --exclude-files="$EXCLUDES" \
             --output-dir $(pwd)/reports/$provider --features="$provider-provider,direct-authenticator" \
             --run-types bins --timeout 3600 -- -c $CONFIG_PATH &
         wait_for_service
@@ -288,7 +314,7 @@ if [ "$PROVIDER_NAME" = "coverage" ]; then
         fi
 
         # Start service
-        RUST_LOG=info cargo +${MSRV} tarpaulin --out Xml --forward --command build --exclude-files="$EXCLUDES" \
+        RUST_LOG=info cargo tarpaulin --out Xml --forward --command build --exclude-files="$EXCLUDES" \
             --output-dir $(pwd)/reports/$provider --features="$provider-provider,direct-authenticator" \
             --run-types bins --timeout 3600 -- -c $CONFIG_PATH &
         wait_for_service
@@ -299,7 +325,7 @@ if [ "$PROVIDER_NAME" = "coverage" ]; then
 
     # Run unit tests
     mkdir -p reports/unit
-    cargo +${MSRV} tarpaulin --tests --out Xml --features=$UNIT_TEST_FEATURES --exclude-files="$EXCLUDES" --output-dir $(pwd)/reports/unit
+    cargo tarpaulin --tests --out Xml --features=$UNIT_TEST_FEATURES --exclude-files="$EXCLUDES" --output-dir $(pwd)/reports/unit
 
     exit 0
 fi
@@ -334,13 +360,10 @@ if [ "$PROVIDER_NAME" = "cargo-check" ]; then
     # - openSUSE Tumbleweed
     # - openSUSE Leap 15.4
 
-    rustup toolchain install ${MSRV}
     # TODO: The "jwt-svid-authenticator" is currently not being used.
-    RUST_BACKTRACE=1 cargo +${MSRV} check --release --features=all-providers,direct-authenticator,unix-peer-credentials-authenticator
+    RUST_BACKTRACE=1 cargo check --release --features=all-providers,direct-authenticator,unix-peer-credentials-authenticator
 
-    # Latest stable
-    rustup toolchain install stable
-    RUST_BACKTRACE=1 cargo +stable check --release $FEATURES
+    RUST_BACKTRACE=1 cargo check --release $FEATURES
 
     # We test that each feature still exist.
     RUST_BACKTRACE=1 cargo check
@@ -375,8 +398,14 @@ fi
 echo "Unit, doc and integration tests"
 RUST_BACKTRACE=1 cargo test $FEATURES
 
+# Ignored unit tests that we want to manually run should be run here!
+if [ "$PROVIDER_NAME" = "tpm" ] || [ "$PROVIDER_NAME" = "all" ]; then
+    RUST_BACKTRACE=1 cargo test $FEATURES -- --ignored test_root_key_check
+fi
+
 # Removing any mappings or on disk keys left over from integration tests
 rm -rf mappings/
+rm -rf kim-mappings/
 rm -f *.psa_its
 
 echo "Start Parsec for end-to-end tests"
