@@ -18,7 +18,6 @@
     overflowing_literals,
     path_statements,
     patterns_in_fns_without_body,
-    private_in_public,
     unconditional_recursion,
     unused,
     unused_allocation,
@@ -39,6 +38,7 @@
 #![allow(clippy::multiple_crate_versions)]
 
 use anyhow::Result;
+use clap::Parser;
 use libc::{getuid, uid_t};
 use log::{info, trace};
 use parsec_service::utils::cli::Opts;
@@ -50,13 +50,12 @@ use std::sync::{
     Arc,
 };
 use std::time::Duration;
-use structopt::StructOpt;
 
 const MAIN_LOOP_DEFAULT_SLEEP: u64 = 10;
 
 fn main() -> Result<()> {
     // Parsing the command line arguments.
-    let opts: Opts = Opts::from_args();
+    let opts: Opts = Opts::parse();
 
     // Register a boolean set to true when the SIGTERM signal is received.
     let kill_signal = Arc::new(AtomicBool::new(false));
@@ -66,7 +65,7 @@ fn main() -> Result<()> {
     let _ = flag::register(SIGINT, kill_signal.clone())?;
     let _ = flag::register(SIGHUP, reload_signal.clone())?;
 
-    let mut config_file = ::std::fs::read_to_string(opts.config.clone()).map_err(|e| {
+    let mut config_file = std::fs::read_to_string(opts.config.clone()).map_err(|e| {
         Error::new(
             e.kind(),
             format!("Failed to read config file from path: {}", opts.config),
@@ -123,7 +122,7 @@ fn main() -> Result<()> {
             drop(listener);
             drop(threadpool);
 
-            config_file = ::std::fs::read_to_string(opts.config.clone()).map_err(|e| {
+            config_file = std::fs::read_to_string(opts.config.clone()).map_err(|e| {
                 Error::new(
                     e.kind(),
                     format!("Failed to read config file from path: {}", opts.config),
@@ -150,7 +149,7 @@ fn main() -> Result<()> {
                 trace!("handle_request egress");
             });
         } else {
-            ::std::thread::sleep(Duration::from_millis(
+            std::thread::sleep(Duration::from_millis(
                 config
                     .core_settings
                     .idle_listener_sleep_duration
