@@ -12,7 +12,7 @@
 //! example, for operating systems having a limit of 255 characters for filenames (Unix systems),
 //! names will be limited to 188 bytes of UTF-8 characters.
 //! For security reasons, only the PARSEC service should have the ability to modify these files.
-use crate::authenticators::{Application, ApplicationIdentity};
+use crate::authenticators::{Application, ApplicationIdentity, Auth, INTERNAL_APP_NAME};
 use crate::utils::config::KeyInfoManagerType;
 
 use super::{KeyIdentity, KeyInfo, ManageKeyInfo, ProviderIdentity};
@@ -45,6 +45,9 @@ use std::{fmt, fs};
 /// Default path where the mapping files will be stored on disk
 pub const DEFAULT_MAPPINGS_PATH: &str = "/var/lib/parsec/mappings";
 
+/// Directory in which the internal keys file will be held
+const INTERNAL_KEYS_PARSEC_DIR: &str = INTERNAL_APP_NAME;
+
 ///Permissions for all directories under database directory
 ///Should only be visible to parsec user
 pub const DIR_PERMISSION: u32 = 0o700;
@@ -53,6 +56,23 @@ pub const DIR_PERMISSION: u32 = 0o700;
 ///Should only be visible to parsec user
 pub const FILE_PERMISSION: u32 = 0o600;
 
+const CORE_PROVIDER: &str = "core";
+const PKCS11_PROVIDER: &str = "pkcs11";
+const MBEDCRYPTO_PROVIDER: &str = "mbedcrypto";
+const TPM_PROVIDER: &str = "tpm";
+const TRUSTEDS_PROVIDER: &str = "trusted-service";
+const CRYPTOAUTH_PROVIDER: &str = "cryptoauthlib";
+
+/// Provider names for internal storage.
+const PROVIDER_NAMES: [&str; 6] = [
+    CORE_PROVIDER,
+    PKCS11_PROVIDER,
+    MBEDCRYPTO_PROVIDER,
+    TPM_PROVIDER,
+    TRUSTEDS_PROVIDER,
+    CRYPTOAUTH_PROVIDER,
+];
+
 /// String wrapper for app names
 #[deprecated(since = "0.9.0", note = "ApplicationIdentity should be used instead.")]
 #[derive(Debug, Clone, Eq, PartialEq, Hash)]
@@ -79,8 +99,8 @@ impl ApplicationName {
 }
 
 #[allow(deprecated)]
-impl std::fmt::Display for ApplicationName {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+impl fmt::Display for ApplicationName {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "{}", self.name)
     }
 }
@@ -152,7 +172,7 @@ impl fmt::Display for KeyTriple {
 impl TryFrom<KeyIdentity> for KeyTriple {
     type Error = String;
 
-    fn try_from(key_identity: KeyIdentity) -> ::std::result::Result<Self, Self::Error> {
+    fn try_from(key_identity: KeyIdentity) -> std::result::Result<Self, Self::Error> {
         let provider_id = match key_identity.provider.uuid().as_str() {
             CoreProvider::PROVIDER_UUID => Ok(ProviderId::Core),
             #[cfg(feature = "cryptoauthlib-provider")]
@@ -173,22 +193,21 @@ impl TryFrom<KeyIdentity> for KeyTriple {
             )),
         }?;
 
-        let app_name = ApplicationName::from_name(key_identity.application().name().clone());
         Ok(KeyTriple {
             provider_id,
-            app_name,
+            app_name: ApplicationName::from_name(key_identity.application().name().to_string()),
             key_name: key_identity.key_name,
         })
     }
 }
 
 #[allow(deprecated)]
-impl TryFrom<(KeyTriple, ProviderIdentity, AuthType)> for KeyIdentity {
+impl TryFrom<(KeyTriple, ProviderIdentity, Auth)> for KeyIdentity {
     type Error = String;
 
     fn try_from(
-        (key_triple, provider_identity, auth_type): (KeyTriple, ProviderIdentity, AuthType),
-    ) -> ::std::result::Result<Self, Self::Error> {
+        (key_triple, provider_identity, auth): (KeyTriple, ProviderIdentity, Auth),
+    ) -> std::result::Result<Self, Self::Error> {
         // Result types required by clippy as Err result has the possibility of not being compiled.
         let provider_uuid = match key_triple.provider_id {
             ProviderId::Core => Ok::<String, Self::Error>(
@@ -214,10 +233,16 @@ impl TryFrom<(KeyTriple, ProviderIdentity, AuthType)> for KeyIdentity {
             _ => Err(format!("Cannot convert from KeyTriple to KeyIdentity.\nProvider \"{}\" is not recognised.\nCould be it does not exist, or Parsec was not compiled with the required provider feature flags.", key_triple.provider_id)),
         }?;
 
+        let app_identity = match auth {
+            Auth::Internal => ApplicationIdentity::new_internal(),
+            Auth::Client(auth_type) => {
+                ApplicationIdentity::new(key_triple.app_name().to_string(), auth_type)
+            }
+        };
         Ok(KeyIdentity {
             provider: ProviderIdentity::new(provider_uuid, provider_identity.name().clone()),
-            application: ApplicationIdentity::new(key_triple.app_name().to_string(), auth_type),
-            key_name: key_triple.key_name.to_string(),
+            application: app_identity,
+            key_name: key_triple.key_name,
         })
     }
 }
@@ -228,6 +253,9 @@ pub struct OnDiskKeyInfoManager {
     /// Internal mapping, used for non-modifying operations.
     #[allow(deprecated)]
     key_store: HashMap<KeyTriple, KeyInfo>,
+    /// Internal mapping with Internal Keys, used for non-modifying operations.
+    #[allow(deprecated)]
+    key_store_internal: HashMap<KeyTriple, KeyInfo>,
     /// Folder where all the key triple to key info mappings are saved. This folder will be created
     /// if it does already exist.
     mappings_dir_path: PathBuf,
@@ -325,6 +353,49 @@ fn os_str_to_provider_id(os_str: &OsStr) -> std::io::Result<ProviderId> {
     }
 }
 
+/// Splits a filename into provider_name + key name (for internal keys only)
+///
+/// # Errors
+///
+/// Returns None if no provider_name was detected
+fn split_provider_key_filename(filename: &str) -> Option<(String, String)> {
+    for name in PROVIDER_NAMES {
+        if filename.starts_with(name) {
+            return Some((name.to_string(), filename.replacen(name, "", 1)));
+        }
+    }
+    None
+}
+
+fn provider_id_to_str(provid: ProviderId) -> &'static str {
+    match provid {
+        ProviderId::Core => CORE_PROVIDER,
+        ProviderId::Pkcs11 => PKCS11_PROVIDER,
+        ProviderId::MbedCrypto => MBEDCRYPTO_PROVIDER,
+        ProviderId::Tpm => TPM_PROVIDER,
+        ProviderId::TrustedService => TRUSTEDS_PROVIDER,
+        ProviderId::CryptoAuthLib => CRYPTOAUTH_PROVIDER,
+    }
+}
+
+fn str_to_provider_id(provid: &str) -> Option<ProviderId> {
+    match provid {
+        CORE_PROVIDER => Some(ProviderId::Core),
+        PKCS11_PROVIDER => Some(ProviderId::Pkcs11),
+        MBEDCRYPTO_PROVIDER => Some(ProviderId::MbedCrypto),
+        TPM_PROVIDER => Some(ProviderId::Tpm),
+        TRUSTEDS_PROVIDER => Some(ProviderId::TrustedService),
+        CRYPTOAUTH_PROVIDER => Some(ProviderId::CryptoAuthLib),
+        _ => None,
+    }
+}
+
+fn prov_and_key_to_str(provider_id: ProviderId, key_name: &str) -> String {
+    let mut p_name = provider_id_to_str(provider_id).to_string();
+    p_name.push_str(key_name);
+    p_name
+}
+
 /// Lists all the directory paths in the given directory path.
 fn list_dirs(path: &Path) -> std::io::Result<Vec<PathBuf>> {
     // read_dir returning an iterator over Result<DirEntry>, there is first a conversion to a path
@@ -371,10 +442,16 @@ impl OnDiskKeyInfoManager {
     /// |---app2/
     /// |   ...
     /// |---appN/
+    /// |---parsec/
+    /// |   |---provider1-INTERNAL1
+    /// |   |---provider2-INTERNAL2
     ///
     /// where the path of a key name from the mappings directory is the key triple (application,
     /// provider, key) and the data inside the key name file is the key info serialised in binary
     /// format.
+    /// The INTERNAL files under the parsec directory stores the information of those keys that are
+    /// internally generated. The file name of the INTERNAL files has as its first component the
+    /// provider name of the provider that generated the internal key.
     /// Each mapping is contained in its own file to prevent the modification of one mapping
     /// impacting the other ones.
     ///
@@ -383,6 +460,7 @@ impl OnDiskKeyInfoManager {
     /// Returns an std::io error if the function failed reading the mapping files.
     fn new(mappings_dir_path: PathBuf, auth_type: AuthType) -> Result<OnDiskKeyInfoManager> {
         let mut key_store = HashMap::new();
+        let mut key_store_internal = HashMap::new();
 
         // Will ignore if the mappings directory already exists.
         fs::create_dir_all(&mappings_dir_path).with_context(|| {
@@ -392,6 +470,48 @@ impl OnDiskKeyInfoManager {
             )
         })?;
 
+        // The INTERNAL files are the only files that on the second level in the directory structure
+        // and store the information of internal keys
+        let parsec_dir = mappings_dir_path.join(INTERNAL_KEYS_PARSEC_DIR);
+        fs::create_dir_all(&parsec_dir)
+            .with_context(|| format!("Failed to create Parsec directory at {:?}", parsec_dir))?;
+        fs::set_permissions(&parsec_dir, Permissions::from_mode(DIR_PERMISSION))?;
+
+        for internal_key_path in list_files(&parsec_dir)?.iter() {
+            let key_name_internal = base64_data_to_string(os_str_to_u8_ref(
+                internal_key_path
+                    .file_name()
+                    .expect("The key name directory path should contain a final component."),
+            )?)
+            .map_err(|e| Error::new(ErrorKind::Other, e))?;
+
+            // If provider name is not recognized, ignore the key.
+            let keytriple = match split_provider_key_filename(&key_name_internal) {
+                None => continue,
+                Some((prov, key_name)) => KeyTriple::new(
+                    ApplicationName::from_name(INTERNAL_KEYS_PARSEC_DIR.to_string()),
+                    // Provider name has been recognized, so if None is returned here something has
+                    // gone terribly wrong.
+                    str_to_provider_id(&prov).unwrap(),
+                    key_name,
+                ),
+            };
+
+            let mut key_info = Vec::new();
+            let mut key_info_file = File::open(internal_key_path).with_context(|| {
+                format!(
+                    "Failed to open Key Info Mappings file at {:?}",
+                    internal_key_path
+                )
+            })?;
+            let _ = key_info_file.read_to_end(&mut key_info)?;
+            let key_info = bincode::deserialize(&key_info[..]).map_err(|e| {
+                format_error!("Error deserializing key info", e);
+                Error::new(ErrorKind::Other, "error deserializing key info")
+            })?;
+            let _ = key_store_internal.insert(keytriple, key_info);
+        }
+
         for app_name_dir_path in list_dirs(&mappings_dir_path)?.iter() {
             for provider_dir_path in list_dirs(app_name_dir_path)?.iter() {
                 for key_name_file_path in list_files(provider_dir_path)?.iter() {
@@ -450,6 +570,7 @@ impl OnDiskKeyInfoManager {
 
         Ok(OnDiskKeyInfoManager {
             key_store,
+            key_store_internal,
             mappings_dir_path,
             auth_type,
         })
@@ -458,38 +579,59 @@ impl OnDiskKeyInfoManager {
     /// Saves the key triple to key info mapping in its own file.
     /// The filename will be `mappings/[APP_NAME]/[PROVIDER_NAME]/[KEY_NAME]` under the same path as the
     /// on-disk manager. It will contain the Key info data.
-    fn save_mapping(&self, key_triple: &KeyTriple, key_info: &KeyInfo) -> std::io::Result<()> {
+    fn save_mapping(
+        &self,
+        key_triple: &KeyTriple,
+        key_info: &KeyInfo,
+        auth: &Auth,
+    ) -> std::io::Result<()> {
         if crate::utils::GlobalConfig::log_error_details() {
             warn!(
                 "Saving Key Triple ({}) mapping to disk.",
                 key_triple.clone()
             );
         }
-        // Create the directories with base64 names.
         let (app_name, prov, key_name) = key_triple_to_base64_filenames(key_triple);
-        let app_dir_path = self.mappings_dir_path.join(app_name);
-        let provider_dir_path = app_dir_path.join(prov);
-        let key_name_file_path = provider_dir_path.join(key_name);
+        let key_name_file_path = match auth {
+            Auth::Internal => {
+                let prov_key_name =
+                    prov_and_key_to_str(key_triple.provider_id, &key_triple.key_name);
+                let prov_key_name =
+                    base64::engine::general_purpose::URL_SAFE.encode(prov_key_name.as_bytes());
+                // INTERNAL_KEYS_PARSEC_DIR has already been created with the necessary permissions
+                self.mappings_dir_path
+                    .join(INTERNAL_KEYS_PARSEC_DIR)
+                    .join(prov_key_name)
+            }
+            Auth::Client(_) => {
+                // Create the directories with base64 names.
+                let app_dir_path = self.mappings_dir_path.join(app_name);
+                let provider_dir_path = app_dir_path.join(prov);
+                let key_name_file_path = provider_dir_path.join(key_name);
+                // Will ignore if they already exist.
+                fs::create_dir_all(&provider_dir_path).map_err(|e| {
+                    format_error!(
+                        format!(
+                            "Failed to create provider directory as {:?}",
+                            &provider_dir_path
+                        ),
+                        e
+                    );
+                    e
+                })?;
+                let dir_permissions = Permissions::from_mode(DIR_PERMISSION);
+                fs::set_permissions(&app_dir_path, dir_permissions.clone())?;
+                fs::set_permissions(&provider_dir_path, dir_permissions)?;
 
-        // Will ignore if they already exist.
-        fs::create_dir_all(&provider_dir_path).map_err(|e| {
-            format_error!(
-                format!(
-                    "Failed to create provider directory as {:?}",
-                    &provider_dir_path
-                ),
-                e
-            );
-            e
-        })?;
-        let dir_permissions = Permissions::from_mode(DIR_PERMISSION);
-        fs::set_permissions(&app_dir_path, dir_permissions.clone())?;
-        fs::set_permissions(&provider_dir_path, dir_permissions)?;
-        if key_name_file_path.exists() {
-            fs::remove_file(&key_name_file_path)?;
-        }
+                if key_name_file_path.exists() {
+                    fs::remove_file(&key_name_file_path)?;
+                }
+                key_name_file_path
+            }
+        };
 
-        let mut mapping_file = fs::File::create(&key_name_file_path).map_err(|e| {
+        // Create the mapping file with the corresponding permissions and write the key information
+        let mut mapping_file = File::create(&key_name_file_path).map_err(|e| {
             error!(
                 "Failed to create Key Info Mapping file at {:?}",
                 key_name_file_path
@@ -507,13 +649,23 @@ impl OnDiskKeyInfoManager {
 
     /// Removes the mapping file.
     /// Will do nothing if the mapping file does not exist.
-    fn delete_mapping(&self, key_triple: &KeyTriple) -> std::io::Result<()> {
+    fn delete_mapping(&self, key_triple: &KeyTriple, auth: &Auth) -> std::io::Result<()> {
         let (app_name, prov, key_name) = key_triple_to_base64_filenames(key_triple);
-        let key_name_file_path = self
-            .mappings_dir_path
-            .join(app_name)
-            .join(prov)
-            .join(key_name);
+        let key_name_file_path = match auth {
+            Auth::Internal => {
+                let prov_key_name = prov_and_key_to_str(key_triple.provider_id, &key_name);
+
+                // INTERNAL_KEYS_PARSEC_DIR has already been created with the necessary permissions
+                self.mappings_dir_path
+                    .join(INTERNAL_KEYS_PARSEC_DIR)
+                    .join(prov_key_name)
+            }
+            Auth::Client(_) => self
+                .mappings_dir_path
+                .join(app_name)
+                .join(prov)
+                .join(key_name),
+        };
         if key_name_file_path.exists() {
             fs::remove_file(key_name_file_path)
         } else {
@@ -532,7 +684,12 @@ impl ManageKeyInfo for OnDiskKeyInfoManager {
         let key_triple = KeyTriple::try_from(key_identity.clone())?;
         // An Option<&Vec<u8>> can not automatically coerce to an Option<&[u8]>, it needs to be
         // done by hand.
-        if let Some(key_info) = self.key_store.get(&key_triple) {
+        let curr_key_store = match key_identity.application().auth() {
+            &Auth::Internal => &self.key_store_internal,
+            _ => &self.key_store,
+        };
+
+        if let Some(key_info) = curr_key_store.get(&key_triple) {
             Ok(Some(key_info))
         } else {
             Ok(None)
@@ -541,17 +698,31 @@ impl ManageKeyInfo for OnDiskKeyInfoManager {
 
     fn get_all(&self, provider_identity: ProviderIdentity) -> Result<Vec<KeyIdentity>, String> {
         let provider_id = ProviderId::try_from(provider_identity.clone())?;
-        let key_triples = self
+        let mut key_identites = Vec::new();
+
+        let key_triples_internal = self
+            .key_store_internal
+            .keys()
+            .filter(|key_triple| key_triple.belongs_to_provider(provider_id));
+
+        for key_triple_internal in key_triples_internal {
+            let key_identity = KeyIdentity::try_from((
+                key_triple_internal.clone(),
+                provider_identity.clone(),
+                Auth::Internal,
+            ))?;
+            key_identites.push(key_identity)
+        }
+        let key_triples_external = self
             .key_store
             .keys()
             .filter(|key_triple| key_triple.belongs_to_provider(provider_id));
 
-        let mut key_identites = Vec::new();
-        for key_triple in key_triples {
+        for key_triple_external in key_triples_external {
             let key_identity = KeyIdentity::try_from((
-                key_triple.clone(),
+                key_triple_external.clone(),
                 provider_identity.clone(),
-                self.auth_type,
+                Auth::Client(self.auth_type),
             ))?;
             key_identites.push(key_identity)
         }
@@ -563,9 +734,13 @@ impl ManageKeyInfo for OnDiskKeyInfoManager {
         key_identity: KeyIdentity,
         key_info: KeyInfo,
     ) -> Result<Option<KeyInfo>, String> {
-        let key_triple = KeyTriple::try_from(key_identity)?;
-        if let Err(err) = self.save_mapping(&key_triple, &key_info) {
+        let key_triple = KeyTriple::try_from(key_identity.clone())?;
+        if let Err(err) =
+            self.save_mapping(&key_triple, &key_info, key_identity.application().auth())
+        {
             Err(err.to_string())
+        } else if key_identity.application().is_internal() {
+            Ok(self.key_store_internal.insert(key_triple, key_info))
         } else {
             Ok(self.key_store.insert(key_triple, key_info))
         }
@@ -573,10 +748,13 @@ impl ManageKeyInfo for OnDiskKeyInfoManager {
 
     fn remove(&mut self, key_identity: &KeyIdentity) -> Result<Option<KeyInfo>, String> {
         let key_triple = KeyTriple::try_from(key_identity.clone())?;
-        if let Err(err) = self.delete_mapping(&key_triple) {
-            Err(err.to_string())
-        } else if let Some(key_info) = self.key_store.remove(&key_triple) {
-            Ok(Some(key_info))
+        if let Err(err) = self.delete_mapping(&key_triple, key_identity.application().auth()) {
+            return Err(err.to_string());
+        }
+        if key_identity.application().is_internal() {
+            Ok(self.key_store_internal.remove(&key_triple))
+        } else if !key_identity.application().is_internal() {
+            Ok(self.key_store.remove(&key_triple))
         } else {
             Ok(None)
         }
@@ -584,7 +762,10 @@ impl ManageKeyInfo for OnDiskKeyInfoManager {
 
     fn exists(&self, key_identity: &KeyIdentity) -> Result<bool, String> {
         let key_triple = KeyTriple::try_from(key_identity.clone())?;
-        Ok(self.key_store.contains_key(&key_triple))
+        match *key_identity.application().auth() {
+            Auth::Internal => Ok(self.key_store_internal.contains_key(&key_triple)),
+            Auth::Client(_) => Ok(self.key_store.contains_key(&key_triple)),
+        }
     }
 }
 
@@ -895,6 +1076,88 @@ mod test {
         fs::remove_dir_all(path).unwrap();
     }
 
+    #[cfg(feature = "mbed-crypto-provider")]
+    #[test]
+    fn create_and_load_internal_keys() {
+        let path = PathBuf::from(env!("OUT_DIR").to_owned() + "/create_and_load_internal_mappings");
+
+        let key_name1 = "😀 Key One 😀".to_string();
+        let key_identity_1 = KeyIdentity::new(
+            ApplicationIdentity::new_internal(),
+            ProviderIdentity::new(
+                CoreProvider::PROVIDER_UUID.to_string(),
+                CoreProvider::DEFAULT_PROVIDER_NAME.to_string(),
+            ),
+            key_name1,
+        );
+        let key_info1 = test_key_info();
+        let key_name2 = "😇 Key Two 😇".to_string();
+        let key_identity_2 = KeyIdentity::new(
+            ApplicationIdentity::new_internal(),
+            ProviderIdentity::new(
+                MbedCryptoProvider::PROVIDER_UUID.to_string(),
+                MbedCryptoProvider::DEFAULT_PROVIDER_NAME.to_string(),
+            ),
+            key_name2,
+        );
+        let key_info2 = KeyInfo {
+            id: vec![0x12, 0x22, 0x32],
+            attributes: test_key_attributes(),
+        };
+
+        {
+            let mut manager = OnDiskKeyInfoManager::new(path.clone(), AuthType::NoAuth).unwrap();
+            let _ = manager
+                .insert(key_identity_1.clone(), key_info1.clone())
+                .unwrap();
+            let _ = manager
+                .insert(key_identity_2.clone(), key_info2.clone())
+                .unwrap();
+        }
+        // The local hashmap is dropped when leaving the inner scope.
+        {
+            let mut manager = OnDiskKeyInfoManager::new(path.clone(), AuthType::NoAuth).unwrap();
+
+            assert_eq!(
+                manager
+                    .get_all(key_identity_1.provider.clone())
+                    .unwrap()
+                    .len(),
+                1
+            );
+            assert_eq!(
+                manager
+                    .get_all(key_identity_2.provider.clone())
+                    .unwrap()
+                    .len(),
+                1
+            );
+
+            // get() should return the key info of the internal key!
+            assert_eq!(&key_info1, manager.get(&key_identity_1).unwrap().unwrap());
+
+            // get() should not work for the same key if it is marked as External!
+            let mut key_identity3 = key_identity_2.clone();
+            key_identity3.application = ApplicationIdentity::new(
+                key_identity_2.application().name().to_string(),
+                AuthType::UnixPeerCredentials,
+            );
+            assert_eq!(None, manager.get(&key_identity3).unwrap());
+
+            assert_eq!(manager.remove(&key_identity_1).unwrap().unwrap(), key_info1);
+            assert_eq!(manager.remove(&key_identity_2).unwrap().unwrap(), key_info2);
+            assert_eq!(
+                manager
+                    .get_all(key_identity_1.provider.clone())
+                    .unwrap()
+                    .len(),
+                0
+            );
+        }
+
+        fs::remove_dir_all(path).unwrap();
+    }
+
     fn new_key_identity(key_name: String) -> KeyIdentity {
         KeyIdentity::new(
             ApplicationIdentity::new("Testing Application 😎".to_string(), AuthType::NoAuth),
