@@ -4,7 +4,7 @@
 //!
 //! For security reasons, only the PARSEC service should have the ability to modify these files.
 use super::{KeyIdentity, KeyInfo, ManageKeyInfo};
-use crate::authenticators::ApplicationIdentity;
+use crate::authenticators::{ApplicationIdentity, Auth, INTERNAL_AUTH_ID};
 use crate::providers::ProviderIdentity;
 use crate::utils::config::KeyInfoManagerType;
 use anyhow::{Context, Result};
@@ -239,15 +239,16 @@ impl SQLiteKeyInfoManager {
         // Deserialize key mappings and store within local key_store HashMap.
         let mut rows = key_mapping_stmt.query(params![])?;
         while let Some(row) = rows.next()? {
+            let auth = match row.get("authenticator_id")? {
+                INTERNAL_AUTH_ID => Auth::Internal,
+                auth_type => Auth::Client(i64_to_auth_type(auth_type).map_err(|e| {
+                    format_error!("Failed to get AuthType from authenticator_id.", e);
+                    let error = Box::new(Error::new(ErrorKind::InvalidData, e));
+                    RusqliteError::FromSqlConversionFailure(64, Integer, error)
+                })?),
+            };
             let key_identity = KeyIdentity::new(
-                ApplicationIdentity::new(
-                    row.get("application_name")?,
-                    i64_to_auth_type(row.get("authenticator_id")?).map_err(|e| {
-                        format_error!("Failed to get AuthType from authenticator_id.", e);
-                        let error = Box::new(Error::new(ErrorKind::InvalidData, e));
-                        RusqliteError::FromSqlConversionFailure(64, Integer, error)
-                    })?,
-                ),
+                ApplicationIdentity::new_with_auth(row.get("application_name")?, auth),
                 ProviderIdentity::new(row.get("provider_uuid")?, row.get("provider_name")?),
                 row.get("key_name")?,
             );
@@ -314,7 +315,7 @@ impl SQLiteKeyInfoManager {
                 (?1, ?2, ?3, ?4, ?5, ?6, ?7, ?8, ?9);
             ",
             params![
-                *key_identity.application().authenticator_id() as u8,
+                key_identity.application().authenticator_id(),
                 key_identity.application().name(),
                 key_identity.provider().uuid(),
                 key_identity.provider().name(),
@@ -345,7 +346,7 @@ impl SQLiteKeyInfoManager {
                 AND `key_name` = ?3
             ",
             params![
-                *key_identity.application().authenticator_id() as u8,
+                key_identity.application().authenticator_id(),
                 key_identity.application().name(),
                 key_identity.key_name(),
             ],
@@ -964,6 +965,94 @@ mod test {
         fs::remove_file(&path).unwrap();
     }
 
+    #[cfg(feature = "mbed-crypto-provider")]
+    #[test]
+    fn create_and_load_internal_keys() {
+        let path = PathBuf::from(
+            env!("OUT_DIR").to_owned()
+                + "/kim/sqlite/create_and_load_mappings_internal_keys.sqlite3",
+        );
+        fs::remove_file(&path).unwrap_or_default();
+
+        let key_name1 = "😀 Key One 😀".to_string();
+        let key_identity_1 = KeyIdentity::new(
+            ApplicationIdentity::new_internal(),
+            ProviderIdentity::new(
+                CoreProvider::PROVIDER_UUID.to_string(),
+                CoreProvider::DEFAULT_PROVIDER_NAME.to_string(),
+            ),
+            key_name1,
+        );
+        let key_info1 = test_key_info();
+
+        let key_name2 = "😇 Key Two 😇".to_string();
+        let key_identity_2 = KeyIdentity::new(
+            ApplicationIdentity::new_internal(),
+            ProviderIdentity::new(
+                MbedCryptoProvider::PROVIDER_UUID.to_string(),
+                MbedCryptoProvider::DEFAULT_PROVIDER_NAME.to_string(),
+            ),
+            key_name2,
+        );
+        let key_info2 = KeyInfo {
+            id: vec![0x12, 0x22, 0x32],
+            attributes: test_key_attributes(),
+        };
+
+        {
+            let mut manager = SQLiteKeyInfoManager::new(path.clone()).unwrap();
+
+            let _ = manager
+                .insert(key_identity_1.clone(), key_info1.clone())
+                .unwrap();
+            let _ = manager
+                .insert(key_identity_2.clone(), key_info2.clone())
+                .unwrap();
+        }
+        // The local hashmap is dropped when leaving the inner scope.
+        {
+            let mut manager = SQLiteKeyInfoManager::new(path.clone()).unwrap();
+
+            assert_eq!(
+                manager
+                    .get_all(key_identity_1.provider.clone())
+                    .unwrap()
+                    .len(),
+                1
+            );
+            assert_eq!(
+                manager
+                    .get_all(key_identity_2.provider.clone())
+                    .unwrap()
+                    .len(),
+                1
+            );
+
+            // get() should return the key info of the internal key!
+            assert_eq!(&key_info1, manager.get(&key_identity_1).unwrap().unwrap());
+
+            // get() should not work for the same key if it is marked as External!
+            let mut key_identity3 = key_identity_2.clone();
+            key_identity3.application = ApplicationIdentity::new(
+                key_identity_2.application().name().to_string(),
+                AuthType::UnixPeerCredentials,
+            );
+            assert_eq!(None, manager.get(&key_identity3).unwrap());
+
+            assert_eq!(manager.remove(&key_identity_1).unwrap().unwrap(), key_info1);
+            assert_eq!(manager.remove(&key_identity_2).unwrap().unwrap(), key_info2);
+            assert_eq!(
+                manager
+                    .get_all(key_identity_1.provider.clone())
+                    .unwrap()
+                    .len(),
+                0
+            );
+        }
+
+        fs::remove_file(&path).unwrap();
+    }
+
     fn new_key_identity(key_name: String) -> KeyIdentity {
         KeyIdentity::new(
             ApplicationIdentity::new("Testing Application 😎".to_string(), AuthType::NoAuth),
