@@ -652,9 +652,8 @@ impl ProviderBuilder {
             None => return Err(Error::new(ErrorKind::InvalidData, "Missing inteface type")),
         };
         Provider::new(
-            self.provider_name.ok_or_else(|| {
-                std::io::Error::new(std::io::ErrorKind::InvalidData, "missing provider name")
-            })?,
+            self.provider_name
+                .ok_or_else(|| Error::new(ErrorKind::InvalidData, "missing provider name"))?,
             self.key_info_store
                 .ok_or_else(|| Error::new(ErrorKind::InvalidData, "missing key info store"))?,
             iface_cfg,
