@@ -26,7 +26,6 @@ use parsec_interface::operations::{list_clients, list_keys, list_providers::Prov
 use parsec_interface::requests::{Opcode, ProviderId, ResponseStatus, Result};
 use parsec_interface::secrecy::{ExposeSecret, SecretString};
 use std::collections::HashSet;
-use std::convert::From;
 use std::convert::TryFrom;
 use std::io::{Error, ErrorKind};
 use std::str::FromStr;
@@ -587,9 +586,8 @@ impl ProviderBuilder {
         };
 
         Ok(Provider::new(
-            self.provider_name.ok_or_else(|| {
-                std::io::Error::new(std::io::ErrorKind::InvalidData, "missing provider name")
-            })?,
+            self.provider_name
+                .ok_or_else(|| Error::new(ErrorKind::InvalidData, "missing provider name"))?,
             self.key_info_store
                 .ok_or_else(|| Error::new(ErrorKind::InvalidData, "missing key info store"))?,
             backend,
