@@ -6,12 +6,16 @@
 //! for their Parsec operations.
 use super::Provide;
 use crate::authenticators::ApplicationIdentity;
-use crate::key_info_managers::KeyInfoManagerClient;
+use crate::key_info_managers::{KeyIdentity, KeyInfoManagerClient};
 use crate::providers::crypto_capability::CanDoCrypto;
 use crate::providers::ProviderIdentity;
 use derivative::Derivative;
 use log::{info, trace};
 use parsec_interface::operations::list_providers::Uuid;
+use parsec_interface::operations::psa_algorithm::Algorithm;
+use parsec_interface::operations::psa_key_attributes::{
+    Attributes, Lifetime, Policy, Type, UsageFlags,
+};
 use parsec_interface::operations::{
     attest_key, can_do_crypto, prepare_key_attestation, psa_asymmetric_decrypt,
     psa_asymmetric_encrypt, psa_destroy_key, psa_export_public_key, psa_generate_key,
@@ -416,9 +420,10 @@ impl ProviderBuilder {
             builder = builder.with_hierarchy_auth(Hierarchy::Endorsement, endorsement_auth);
             self.endorsement_hierarchy_auth.zeroize();
         }
-        Ok(Provider::new(
+
+        let built_provider = Provider::new(
             self.provider_name.ok_or_else(|| {
-                std::io::Error::new(std::io::ErrorKind::InvalidData, "missing provider name")
+                std::io::Error::new(ErrorKind::InvalidData, "missing provider name")
             })?,
             self.key_info_store.ok_or_else(|| {
                 std::io::Error::new(ErrorKind::InvalidData, "missing key info store")
@@ -427,6 +432,195 @@ impl ProviderBuilder {
                 format_error!("Error creating TSS Transient Object Context", e);
                 std::io::Error::new(ErrorKind::InvalidData, "failed initializing TSS context")
             })?,
-        ))
+        );
+
+        // Get the root key from the key store
+        let root_key_identity = KeyIdentity::new(
+            ApplicationIdentity::new_internal(),
+            built_provider.provider_identity.clone(),
+            String::from("RootKeyTPM"),
+        );
+        let key_is_stored = match built_provider
+            .key_info_store
+            .does_not_exist(&root_key_identity)
+        {
+            Ok(()) => false,
+            Err(ResponseStatus::PsaErrorAlreadyExists) => true,
+            Err(e) => Err(e).map_err(|e| {
+                format_error!("Failure accessing Key Info Manager", e);
+                std::io::Error::new(ErrorKind::InvalidData, "Key existence check failed")
+            })?,
+        };
+
+        if key_is_stored {
+            let stored_root_key_name: Vec<u8> = built_provider
+                .key_info_store
+                .get_key_id(&root_key_identity)
+                .map_err(|e| {
+                    format_error!("Error getting Key Identities from the Key Info Store", e);
+                    std::io::Error::new(ErrorKind::InvalidData, "failed getting Key Identities")
+                })?;
+            // Check if the stored public part coincides with the one in the context
+            let mut esapi_context = built_provider
+                .esapi_context
+                .lock()
+                .expect("ESAPI Context lock poisoned");
+
+            let root_key_name = esapi_context.get_root_key_name().map_err(|e| {
+                format_error!("Error getting the Root Key's name", e);
+                std::io::Error::new(ErrorKind::InvalidData, "failed getting Root Key's Name")
+            })?;
+
+            if root_key_name.value().to_vec() != stored_root_key_name {
+                let e = std::io::Error::new(
+                    ErrorKind::InvalidData,
+                    "Obtained Root Key name does not coincide with the stored one",
+                );
+                format_error!("Error when verifying the Root Key's Name", e);
+                return Err(e);
+            }
+        } else {
+            let mut esapi_context = built_provider
+                .esapi_context
+                .lock()
+                .expect("ESAPI Context lock poisoned");
+
+            let root_key_name = esapi_context.get_root_key_name().map_err(|e| {
+                format_error!("Error getting the the Root Key's Name", e);
+                std::io::Error::new(ErrorKind::InvalidData, "failed getting Root Key's Name")
+            })?;
+
+            let attributes = Attributes {
+                lifetime: Lifetime::Persistent,
+                key_type: Type::RsaPublicKey,
+                bits: ROOT_KEY_SIZE as usize,
+                policy: Policy {
+                    // Internal key, usage_flags information is not relevant
+                    usage_flags: UsageFlags::default(),
+                    // Internal key, permitted_algorithms information is not relevant
+                    permitted_algorithms: Algorithm::None,
+                },
+            };
+
+            built_provider
+                .key_info_store
+                .insert_key_info(
+                    root_key_identity,
+                    &(root_key_name.value().to_vec()),
+                    attributes,
+                )
+                .map_err(|_| {
+                    std::io::Error::new(
+                        ErrorKind::InvalidData,
+                        "Failed to insert Key Info in the Key Store",
+                    )
+                })?;
+        }
+
+        Ok(built_provider)
+    }
+}
+
+#[cfg(test)]
+mod test {
+    use crate::key_info_managers::{KeyIdentity, KeyInfoManagerFactory};
+    use crate::providers::tpm::ROOT_KEY_SIZE;
+    use crate::providers::tpm::{Provider, ProviderBuilder};
+    use crate::providers::ApplicationIdentity;
+    use crate::providers::ProviderIdentity;
+    use crate::utils::config::{KeyInfoManagerConfig, KeyInfoManagerType};
+    use parsec_interface::operations::psa_algorithm::Algorithm;
+    use parsec_interface::operations::psa_key_attributes::{
+        Attributes, Lifetime, Policy, Type, UsageFlags,
+    };
+    use parsec_interface::requests::AuthType;
+
+    // This test can only be run with a TPM already available in your system.
+    // Ignored unless specified
+    #[ignore]
+    #[test]
+    fn test_root_key_check() {
+        // Test configuration should be changed to adapt to your system's TPM
+        let tcti = "mssim:host=127.0.0.1,port=2321";
+        let owner_hierarchy_auth = "hex:74706d5f70617373";
+        let endorsement_hierarchy_auth = "str:endorsement_pass".to_string();
+
+        let provider_identity =
+            ProviderIdentity::new(Provider::PROVIDER_UUID.to_string(), "Tpm".to_string());
+        let kim_config = KeyInfoManagerConfig {
+            name: "sqlite-manager".to_string(),
+            manager_type: KeyInfoManagerType::SQLite,
+            store_path: None,
+            sqlite_db_path: Some(
+                "./kim-mappings/sqlite/sqlite-key-info-manager.sqlite3".to_string(),
+            ),
+        };
+
+        let kim_factory = KeyInfoManagerFactory::new(&kim_config, AuthType::NoAuth).unwrap();
+        // Builds the tpm provider and inserts an internal key
+        {
+            let builder = ProviderBuilder::new()
+                .with_key_info_store(kim_factory.build_client(provider_identity.clone()))
+                .with_tcti(tcti)
+                .with_provider_name("Tpm".to_string())
+                .with_owner_hierarchy_auth(owner_hierarchy_auth.to_string())
+                .with_endorsement_hierarchy_auth(endorsement_hierarchy_auth.clone());
+
+            unsafe {
+                let _ = builder.build().unwrap();
+            }
+        }
+
+        // Builds the tpm provider, checking that the internally stored key matches
+        // the newly generated one.
+        // Then, it modifies the key information inside so that the next check fails.
+        {
+            let builder = ProviderBuilder::new()
+                .with_key_info_store(kim_factory.build_client(provider_identity.clone()))
+                .with_tcti(tcti)
+                .with_provider_name("Tpm".to_string())
+                .with_owner_hierarchy_auth(owner_hierarchy_auth.to_string())
+                .with_endorsement_hierarchy_auth(endorsement_hierarchy_auth.clone());
+            // Reads the key and verifies the created one vs the stored one
+
+            let built_provider = unsafe { builder.build().unwrap() };
+
+            // replace the key info from the internal key with some incorrect information
+            let root_key_identity = KeyIdentity::new(
+                ApplicationIdentity::new_internal(),
+                built_provider.provider_identity.clone(),
+                String::from("RootKeyTPM"),
+            );
+            let attributes = Attributes {
+                lifetime: Lifetime::Persistent,
+                key_type: Type::RsaPublicKey,
+                bits: ROOT_KEY_SIZE as usize,
+                policy: Policy {
+                    // Internal key, usage_flags information is not relevant
+                    usage_flags: UsageFlags::default(),
+                    // Internal key, permitted_algorithms information is not relevant
+                    permitted_algorithms: Algorithm::None,
+                },
+            };
+
+            let test_material: Vec<u8> = vec![1, 2];
+
+            built_provider
+                .key_info_store
+                .replace_key_info(root_key_identity, &test_material, attributes)
+                .unwrap();
+        }
+
+        // Should fail as the newly replaced key information does not match
+        // the previously created one.
+        {
+            let builder = ProviderBuilder::new()
+                .with_key_info_store(kim_factory.build_client(provider_identity))
+                .with_tcti(tcti)
+                .with_provider_name("Tpm".to_string())
+                .with_owner_hierarchy_auth(owner_hierarchy_auth.to_string())
+                .with_endorsement_hierarchy_auth(endorsement_hierarchy_auth);
+            assert!(unsafe { builder.build().is_err() });
+        }
     }
 }
