@@ -5,8 +5,9 @@
 // WARNING: This file should be only updated in a non-breaking way. CLI flags should not be
 // removed, new flags should be tested.
 // See https://github.com/parallaxsecond/parsec/issues/392 for details.
+#![allow(unused_qualifications)]
 
-use structopt::StructOpt;
+use clap::Parser;
 
 /// Parsec is the Platform AbstRaction for SECurity, a new open-source initiative to provide a
 /// common API to secure services in a platform-agnostic way.
@@ -17,7 +18,7 @@ use structopt::StructOpt;
 /// Most of Parsec configuration comes from its configuration file.
 /// Please check the documentation to find more about configuration:
 /// https://parallaxsecond.github.io/parsec-book/user_guides/configuration.html
-#[derive(StructOpt, Debug)]
+#[derive(Parser, Debug)]
 pub struct Opts {
     /// Sets the configuration file path
     #[structopt(short, long, default_value = "config.toml")]
